/*:
 * @plugindesc プレイヤーキャラの位置や任意の位置・任意のIDのイベントを起動します。(HzStartEvent.js改変)
 * @author オリジナル：hiz & 改変：にゃたま
 *
 * @help
 * このプラグインは以下の機能を持ちます。
 * ・プレイヤーキャラの位置のや指定した座標の周辺イベントを起動します。必ず起動したいイベントのメモ欄に<SE>を書いて下さい。
 *　 ※ 移動床などで移動後に続けてイベントを起動したい場合等に利用できます。
 * ・プレイヤーキャラからの相対座標又は絶対座標で指定した位置のイベントを起動します。
 * 　※ 選択肢によってイベントを起動する場合などに利用できます。
 * ・指定IDのイベントを起動します。
 * 　※ 汎用的に利用できます。IDには変数も指定できます。
 *
 * 注意点
 *   ・起動されるイベントのページは通常の起動と同じく、出現条件を満たす最も大きいページ数のページとなります。
 *   ・起動させたいイベントにはメモ欄に<SE>を書くことで起動できるようになります。
 *
 *   ・イベントは、現在のイベント処理が完了した後に実行されます。
 *
 * プラグインコマンド
 *   StartEvent                         # プレイヤーキャラの位置のイベントを起動します。
 *   StartEvent rel [x] [y]             # プレイヤーキャラから右にXマス、下にYマス移動した位置にあるイベントを起動します。
 *   StartEvent rel [x] [y] [zone]      # プレイヤーキャラから右にXマス、下にYマス移動した位置にあるイベントから
 *                                        # zoneの範囲までのイベントを起動します。
 *   StartEvent abs [x] [y]             # 座標（X、Y）の位置にあるイベントを起動します。
 *   StartEvent abs [x] [y] [zone]      # 座標（X、Y）の位置からzoneの範囲までのイベントを起動します。
 *   StartEvent id [id]                 # イベントID[id]のイベントを起動します。
 *   StartEvent id [zone]               # イベントID[id]のイベントからzoneの範囲までのイベントを起動します。
 *
 *  例）
 *   StartEvent                         # プレイヤーキャラの位置のイベントを起動します。
 *   StartEvent rel 3 2                 # プレイヤーキャラから右に3マス、下に2マス移動した位置にあるイベントを起動します。
 *   StartEvent abs 3 4                 # 座標（3, 4）の位置にあるイベントを起動します。
 *   StartEvent id 2                    # イベントID2のイベントを起動します。
 *   StartEvent id \V[1]                # 変数1のイベントIDのイベントを起動します。
 *
 * ※ パラメータには制御文字も指定できます。
 * ※ 「startevent」は小文字大文字どちらでもOKです。
 */

function NYA_StartEvent() {}

(function() {
    
  function convertEscape(txt) {
    return Window_Base.prototype.convertEscapeCharacters(txt)
  };

  var _Game_Interpreter_pluginCommand = Game_Interpreter.prototype.pluginCommand;
  Game_Interpreter.prototype.pluginCommand = function(command, args) {
      
    _Game_Interpreter_pluginCommand.call(this, command, args);
      
    if (command.toUpperCase() === 'STARTEVENT') {
      if (args.length == 0) {
        // プレイヤーキャラの位置のイベントを起動
        var x = $gamePlayer.x;
        var y = $gamePlayer.y;
        var event = getEventByPos(x, y);
        startEvent(event);
      } else {
        var type = args[0];
        if (type.toUpperCase() === 'REL') {
          //  プレイヤーキャラから右にXマス、下にYマス移動した位置にあるイベントを起動
          var dx = Number(convertEscape(args[1]));
          var dy = Number(convertEscape(args[2]));
          var x = $gamePlayer.x + dx;
          var y = $gamePlayer.y + dy;
          if(args[3]){
            var zone = Number(convertEscape(args[3]));
            startEventByPosZone(x,y,zone);
          }else{
            startEventByPos(x,y);
          }
        } else if (type.toUpperCase() === 'ABS') {
            // 指定した座標の位置にあるイベントを起動
          var x = Number(convertEscape(args[1]));
          var y = Number(convertEscape(args[2]));
          if(args[3]){
            var zone = Number(convertEscape(args[3]));
            startEventByPosZone(x,y,zone);
          }else{
            startEventByPos(x,y);
          }
        } else if (type.toUpperCase() === 'ID') {
          // イベントID[id]のイベントを起動
          var id = Number(convertEscape(args[1]));
          if(args[2]){
            var zone = Number(convertEscape(args[2]));
            startEventByPosZone($gameMap.event(id).x, $gameMap.event(id).y, zone);
          }else{
            var event = $gameMap.event(id);
            startEvent(event);
          }
        }
      }
    }
  }
  function startEventByPosZone(x, y, zone){
    // ゾーンの指定がある場合
    for (var i=x-zone;i<x+zone+1;i++){
      for (var j=y-zone;j<y+zone+1;j++){
          // 座標（X、Y）の位置にあるイベントを起動
          var event = getEventByPos(i, j);
          startEvent(event);
      }
    }
  }
  function startEventByPos(x, y){
    // ゾーンの指定がない場合
    // 座標（X、Y）の位置にあるイベントを起動
    var event = getEventByPos(x, y);
    startEvent(event);
  }
  
  /**
   * 座標(x, y)のイベントのIDを取得
   */
  function getEventByPos(x, y) {
    var targets = $gameMap.events().filter(function(event) {
      return event.x == x && event.y == y;
    });
    if (targets.length === 0) return null;
    return targets[0];
  }

  /**
   * 指定イベントの起動
   */
  function startEvent(event) {
    if (event == null) return;          //イベントそのものが無いときは起動しない
    if (!event.event().meta.SE) return; //イベントのメモ欄に<SE>タグの無いのは起動しない
    if (event.erased()) return;         //イベントの一時消去により削除されたイベントを起動しない
    event.start();
  }
    
  Game_Event.prototype.erased = function() {
    return this._erased;
  };
    
})();
